// Oldschoolengine CPU routines in IWRAM
//
// To get benefit from local variables, macros are used instead of inline
// functions.
//
// NOTE: for speed optimization, effective address calculation does not
// wrap the zeropage or even the memory correctly! But wellbehaved code
// doesn't rely on that anyway :)

#define MEM(address) (cpumem[address])
#define LO() (MEM(pc))
#define HI() (MEM(pc+1))
#define FETCH() (MEM(pc++))
#define SETPC(newpc) (pc = (newpc))
#define PUSH(data) (MEM(0x100 + (sp--)) = (data))
#define POP() (MEM(0x100 + (++sp)))

#define IMMEDIATE() (LO())
#define ABSOLUTE() (LO() | (HI() << 8))
#define ABSOLUTEX() ((LO() | (HI() << 8)) + x)
#define ABSOLUTEY() ((LO() | (HI() << 8)) + y)
#define ZEROPAGE() (LO())
#define ZEROPAGEX() (LO() + x)
#define ZEROPAGEY() (LO() + y)
#define INDIRECTX() (MEM((LO() + x) & 0xff) | (MEM(LO() + x + 1) << 8))
#define INDIRECTY() ((MEM(LO()) | (MEM(LO() + 1) << 8)) + y)

#define WRITE(address)                  \
{                                       \
  cpuwritemap[(address) >> 6] = 1;      \
}

#define BRANCH()                        \
{                                       \
  temp = FETCH();                       \
  if (temp < 0x80)                      \
    SETPC(pc + temp);                   \
  else                                  \
    SETPC(pc + temp - 0x100);           \
}

#define SETFLAGS(data)                  \
{                                       \
  if (!(data))                          \
    flags = (flags & ~FN) | FZ;         \
  else                                  \
    flags = (flags & ~(FN|FZ)) |        \
    ((data) & FN);                      \
}

#define ASSIGNSETFLAGS(dest, data)      \
{                                       \
  dest = data;                          \
  if (!dest)                            \
    flags = (flags & ~FN) | FZ;         \
  else                                  \
    flags = (flags & ~(FN|FZ)) |        \
    (dest & FN);                        \
}

#define ADC(data)                                                        \
{                                                                        \
    u32 tempval = data;                                                  \
                                                                         \
    if (flags & FD)                                                      \
    {                                                                    \
        temp = (a & 0xf) + (tempval & 0xf) + (flags & FC);               \
        if (temp > 0x9)                                                  \
            temp += 0x6;                                                 \
        if (temp <= 0x0f)                                                \
            temp = (temp & 0xf) + (a & 0xf0) + (tempval & 0xf0);         \
        else                                                             \
            temp = (temp & 0xf) + (a & 0xf0) + (tempval & 0xf0) + 0x10;  \
        if (!((a + tempval + (flags & FC)) & 0xff))                      \
            flags |= FZ;                                                 \
        else                                                             \
            flags &= ~FZ;                                                \
        if (temp & 0x80)                                                 \
            flags |= FN;                                                 \
        else                                                             \
            flags &= ~FN;                                                \
        if (((a ^ temp) & 0x80) && !((a ^ tempval) & 0x80))              \
            flags |= FV;                                                 \
        else                                                             \
            flags &= ~FV;                                                \
        if ((temp & 0x1f0) > 0x90) temp += 0x60;                         \
        if ((temp & 0xff0) > 0xf0)                                       \
            flags |= FC;                                                 \
        else                                                             \
            flags &= ~FC;                                                \
    }                                                                    \
    else                                                                 \
    {                                                                    \
        temp = tempval + a + (flags & FC);                               \
        SETFLAGS(temp & 0xff);                                           \
        if (!((a ^ tempval) & 0x80) && ((a ^ temp) & 0x80))              \
            flags |= FV;                                                 \
        else                                                             \
            flags &= ~FV;                                                \
        if (temp > 0xff)                                                 \
            flags |= FC;                                                 \
        else                                                             \
            flags &= ~FC;                                                \
    }                                                                    \
    a = temp;                                                            \
}

#define SBC(data)                                                        \
{                                                                        \
    u32 tempval = data;                                                  \
    temp = a - tempval - ((flags & FC) ^ FC);                            \
                                                                         \
    if (flags & FD)                                                      \
    {                                                                    \
        u32 tempval2;                                                    \
        tempval2 = (a & 0xf) - (tempval & 0xf) - ((flags & FC) ^ FC);    \
        if (tempval2 & 0x10)                                             \
            tempval2 = ((tempval2 - 6) & 0xf) | ((a & 0xf0) - (tempval   \
            & 0xf0) - 0x10);                                             \
        else                                                             \
            tempval2 = (tempval2 & 0xf) | ((a & 0xf0) - (tempval         \
            & 0xf0));                                                    \
        if (tempval2 & 0x100)                                            \
            tempval2 -= 0x60;                                            \
        if (temp < 0x100)                                                \
            flags |= FC;                                                 \
        else                                                             \
            flags &= ~FC;                                                \
        SETFLAGS(temp & 0xff);                                           \
        if (((a ^ temp) & 0x80) && ((a ^ tempval) & 0x80))               \
            flags |= FV;                                                 \
        else                                                             \
            flags &= ~FV;                                                \
        a = tempval2;                                                    \
    }                                                                    \
    else                                                                 \
    {                                                                    \
        SETFLAGS(temp & 0xff);                                           \
        if (temp < 0x100)                                                \
            flags |= FC;                                                 \
        else                                                             \
            flags &= ~FC;                                                \
        if (((a ^ temp) & 0x80) && ((a ^ tempval) & 0x80))               \
            flags |= FV;                                                 \
        else                                                             \
            flags &= ~FV;                                                \
        a = temp;                                                        \
    }                                                                    \
}

#define CMP(src, data)                  \
{                                       \
  temp = (src - data) & 0xff;           \
                                        \
  flags = (flags & ~(FC|FN|FZ)) |       \
          (temp & FN);                  \
                                        \
  if (!temp) flags |= FZ;               \
  if (src >= data) flags |= FC;         \
}

#define ASL(data)                       \
{                                       \
  temp = data;                          \
  temp <<= 1;                           \
  if (temp & 0x100) flags |= FC;        \
  else flags &= ~FC;                    \
  ASSIGNSETFLAGS(data, temp);           \
}

#define LSR(data)                       \
{                                       \
  temp = data;                          \
  if (temp & 1) flags |= FC;            \
  else flags &= ~FC;                    \
  temp >>= 1;                           \
  ASSIGNSETFLAGS(data, temp);           \
}

#define ROL(data)                       \
{                                       \
  temp = data;                          \
  temp <<= 1;                           \
  if (flags & FC) temp |= 1;            \
  if (temp & 0x100) flags |= FC;        \
  else flags &= ~FC;                    \
  ASSIGNSETFLAGS(data, temp);           \
}

#define ROR(data)                       \
{                                       \
  temp = data;                          \
  if (flags & FC) temp |= 0x100;        \
  if (temp & 1) flags |= FC;            \
  else flags &= ~FC;                    \
  temp >>= 1;                           \
  ASSIGNSETFLAGS(data, temp);           \
}

#define DEC(data)                       \
{                                       \
  temp = data - 1;                      \
  ASSIGNSETFLAGS(data, temp);           \
}

#define INC(data)                       \
{                                       \
  temp = data + 1;                      \
  ASSIGNSETFLAGS(data, temp);           \
}

#define EOR(data)                       \
{                                       \
  a ^= data;                            \
  SETFLAGS(a);                          \
}

#define ORA(data)                       \
{                                       \
  a |= data;                            \
  SETFLAGS(a);                          \
}

#define AND(data)                       \
{                                       \
  a &= data;                            \
  SETFLAGS(a)                           \
}

#define BIT(data)                       \
{                                       \
  flags = (flags & ~(FN|FV)) |          \
          (data & (FN|FV));             \
  if (!(data & a)) flags |= FZ;         \
  else flags &= ~FZ;                    \
}

extern u16 blittersrc;
extern u16 blitterdest;
extern char filename[];

CODE_IN_IWRAM int runcpu(u16 newpc, u8 newa, u8 newx, u8 newy, u8 newsp)
{
  u8 a = newa;
  u8 x = newx;
  u8 y = newy;
  u16 pc = newpc;
  u8 sp = newsp;
  u8 flags = 0;
  u32 temp;

  while(1)
  {
  switch(FETCH())
  {
  	case 0x02:
    return 0;

  	case 0x69:
  	ADC(IMMEDIATE());
  	pc++;
  	break;

  	case 0x65:
  	ADC(MEM(ZEROPAGE()));
  	pc++;
  	break;

  	case 0x75:
  	ADC(MEM(ZEROPAGEX()));
  	pc++;
    break;

    case 0x6d:
  	ADC(MEM(ABSOLUTE()));
  	pc += 2;
    break;

    case 0x7d:
  	ADC(MEM(ABSOLUTEX()));
   	pc += 2;
    break;

    case 0x79:
  	ADC(MEM(ABSOLUTEY()));
  	pc += 2;
    break;

    case 0x61:
  	ADC(MEM(INDIRECTX()));
  	pc++;
    break;

    case 0x71:
  	ADC(MEM(INDIRECTY()));
  	pc++;
    break;

  	case 0x29:
    AND(IMMEDIATE());
  	pc++;
  	break;

  	case 0x25:
  	AND(MEM(ZEROPAGE()));
  	pc++;
  	break;

  	case 0x35:
  	AND(MEM(ZEROPAGEX()));
  	pc++;
    break;

    case 0x2d:
    AND(MEM(ABSOLUTE()));
  	pc += 2;
    break;

    case 0x3d:
    AND(MEM(ABSOLUTEX()));
  	pc += 2;
    break;

    case 0x39:
    AND(MEM(ABSOLUTEY()));
  	pc += 2;
    break;

    case 0x21:
    AND(MEM(INDIRECTX()));
  	pc++;
    break;

    case 0x31:
    AND(MEM(INDIRECTY()));
  	pc++;
    break;

    case 0x0a:
    ASL(a);
    break;

    case 0x06:
    ASL(MEM(ZEROPAGE()));
    pc++;
    break;

    case 0x16:
    ASL(MEM(ZEROPAGEX()));
    pc++;
    break;

    case 0x0e:
    ASL(MEM(ABSOLUTE()));
    pc += 2;
    break;

    case 0x1e:
    ASL(MEM(ABSOLUTEX()));
    pc += 2;
    break;

    case 0x90:
    if (!(flags & FC)) BRANCH()
    else pc++;
    break;

    case 0xb0:
    if (flags & FC) BRANCH()
    else pc++;
    break;

    case 0xf0:
    if (flags & FZ) BRANCH()
    else pc++;
    break;

    case 0x24:
    BIT(MEM(ZEROPAGE()));
    pc++;
    break;

    case 0x2c:
    BIT(MEM(ABSOLUTE()));
    pc += 2;
    break;

    case 0x30:
    if (flags & FN) BRANCH()
    else pc++;
    break;

    case 0xd0:
    if (!(flags & FZ)) BRANCH()
    else pc++;
    break;

    case 0x10:
    if (!(flags & FN)) BRANCH()
    else pc++;
    break;

    case 0x50:
    if (!(flags & FV)) BRANCH()
    else pc++;
    break;

    case 0x70:
    if (flags & FV) BRANCH()
    else pc++;
    break;

    case 0x18:
    flags &= ~FC;
    break;

    case 0xd8:
    flags &= ~FD;
    break;

    case 0x58:
    flags &= ~FI;
    break;

    case 0xb8:
    flags &= ~FV;
    break;

  	case 0xc9:
  	CMP(a, IMMEDIATE());
    pc++;
  	break;

  	case 0xc5:
  	CMP(a, MEM(ZEROPAGE()));
    pc++;
  	break;

  	case 0xd5:
  	CMP(a, MEM(ZEROPAGEX()));
    pc++;
    break;

    case 0xcd:
  	CMP(a, MEM(ABSOLUTE()));
    pc += 2;
    break;

    case 0xdd:
  	CMP(a, MEM(ABSOLUTEX()));
    pc += 2;
    break;

    case 0xd9:
  	CMP(a, MEM(ABSOLUTEY()));
    pc += 2;
    break;

    case 0xc1:
    CMP(a, MEM(INDIRECTX()));
    pc++;
    break;

    case 0xd1:
    CMP(a, MEM(INDIRECTY()));
    pc++;
    break;

    case 0xe0:
    CMP(x, IMMEDIATE());
    pc++;
    break;

    case 0xe4:
    CMP(x, MEM(ZEROPAGE()));
    pc++;
    break;

    case 0xec:
  	CMP(x, MEM(ABSOLUTE()));
    pc += 2;
    break;

    case 0xc0:
    CMP(y, IMMEDIATE());
    pc++;
    break;

    case 0xc4:
    CMP(y, MEM(ZEROPAGE()));
    pc++;
    break;

    case 0xcc:
  	CMP(y, MEM(ABSOLUTE()));
    pc += 2;
    break;

    case 0xc6:
    DEC(MEM(ZEROPAGE()));
    WRITE(ZEROPAGE());
    pc++;
    break;

    case 0xd6:
    DEC(MEM(ZEROPAGEX()));
    WRITE(ZEROPAGEX());
    pc++;
    break;

    case 0xce:
    DEC(MEM(ABSOLUTE()));
    WRITE(ABSOLUTE());
    pc += 2;
    break;

    case 0xde:
    DEC(MEM(ABSOLUTEX()));
    WRITE(ABSOLUTEX());
    pc += 2;
    break;

    case 0xca:
    x--;
    SETFLAGS(x);
    break;

    case 0x88:
    y--;
    SETFLAGS(y);
    break;

  	case 0x49:
  	EOR(IMMEDIATE());
    pc++;
  	break;

  	case 0x45:
  	EOR(MEM(ZEROPAGE()));
  	pc++;
  	break;

  	case 0x55:
  	EOR(MEM(ZEROPAGEX()));
  	pc++;
    break;

    case 0x4d:
    EOR(MEM(ABSOLUTE()));
    pc += 2;
    break;

    case 0x5d:
    EOR(MEM(ABSOLUTEX()));
    pc += 2;
    break;

    case 0x59:
    EOR(MEM(ABSOLUTEY()));
    pc += 2;
    break;

    case 0x41:
  	EOR(MEM(INDIRECTX()));
  	pc++;
    break;

    case 0x51:
  	EOR(MEM(INDIRECTY()));
  	pc++;
    break;

    case 0xe6:
    INC(MEM(ZEROPAGE()));
    WRITE(ZEROPAGE());
    pc++;
    break;

    case 0xf6:
    INC(MEM(ZEROPAGEX()));
    WRITE(ZEROPAGEX());
    pc++;
    break;

    case 0xee:
    INC(MEM(ABSOLUTE()));
    WRITE(ABSOLUTE());
    pc += 2;
    break;

    case 0xfe:
    INC(MEM(ABSOLUTEX()));
    WRITE(ABSOLUTEX());
    pc += 2;
    break;

    case 0xe8:
    x++;
    SETFLAGS(x);
    break;

    case 0xc8:
    y++;
    SETFLAGS(y);
    break;

    case 0x20:
    PUSH((pc+1) >> 8);
    PUSH((pc+1) & 0xff);
    pc = ABSOLUTE();
    break;

    case 0x4c:
    pc = ABSOLUTE();
    break;

    case 0x6c:
    {
      u16 adr = ABSOLUTE();
      pc = (MEM(adr) | (MEM(((adr + 1) & 0xff) | (adr & 0xff00)) << 8));
    }
    break;

  	case 0xa9:
  	ASSIGNSETFLAGS(a, IMMEDIATE());
    pc++;
  	break;

  	case 0xa5:
  	ASSIGNSETFLAGS(a, MEM(ZEROPAGE()));
    pc++;
  	break;

  	case 0xb5:
  	ASSIGNSETFLAGS(a, MEM(ZEROPAGEX()));
    pc++;
    break;

    case 0xad:
  	ASSIGNSETFLAGS(a, MEM(ABSOLUTE()));
    pc += 2;
    break;

    case 0xbd:
  	ASSIGNSETFLAGS(a, MEM(ABSOLUTEX()));
    pc += 2;
    break;

    case 0xb9:
  	ASSIGNSETFLAGS(a, MEM(ABSOLUTEY()));
    pc += 2;
    break;

    case 0xa1:
  	ASSIGNSETFLAGS(a, MEM(INDIRECTX()));
    pc++;
    break;

    case 0xb1:
  	ASSIGNSETFLAGS(a, MEM(INDIRECTY()));
    pc++;
    break;

    case 0xa2:
    ASSIGNSETFLAGS(x, IMMEDIATE());
    pc++;
    break;

    case 0xa6:
  	ASSIGNSETFLAGS(x, MEM(ZEROPAGE()));
    pc++;
    break;

    case 0xb6:
  	ASSIGNSETFLAGS(x, MEM(ZEROPAGEY()));
    pc++;
    break;

    case 0xae:
  	ASSIGNSETFLAGS(x, MEM(ABSOLUTE()));
    pc += 2;
    break;

    case 0xbe:
  	ASSIGNSETFLAGS(x, MEM(ABSOLUTEY()));
    pc += 2;
    break;

    case 0xa0:
    ASSIGNSETFLAGS(y, IMMEDIATE());
    pc++;
    break;

    case 0xa4:
  	ASSIGNSETFLAGS(y, MEM(ZEROPAGE()));
    pc++;
    break;

    case 0xb4:
  	ASSIGNSETFLAGS(y, MEM(ZEROPAGEX()));
    pc++;
    break;

    case 0xac:
  	ASSIGNSETFLAGS(y, MEM(ABSOLUTE()));
    pc += 2;
    break;

    case 0xbc:
  	ASSIGNSETFLAGS(y, MEM(ABSOLUTEX()));
    pc += 2;
    break;

    case 0x4a:
    LSR(a);
    break;

    case 0x46:
    LSR(MEM(ZEROPAGE()));
    WRITE(ZEROPAGE());
    pc++;
    break;

    case 0x56:
    LSR(MEM(ZEROPAGEX()));
    WRITE(ZEROPAGEX());
    pc++;
    break;

    case 0x4e:
    LSR(MEM(ABSOLUTE()));
    WRITE(ABSOLUTE());
    pc += 2;
    break;

    case 0x5e:
    LSR(MEM(ABSOLUTEX()));
    WRITE(ABSOLUTEX());
    pc += 2;
    break;

    case 0xea:
    break;

  	case 0x09:
  	ORA(IMMEDIATE());
    pc++;
  	break;

  	case 0x05:
  	ORA(MEM(ZEROPAGE()));
    pc++;
  	break;

  	case 0x15:
  	ORA(MEM(ZEROPAGEX()));
    pc++;
    break;

    case 0x0d:
  	ORA(MEM(ABSOLUTE()));
    pc += 2;
    break;

    case 0x1d:
  	ORA(MEM(ABSOLUTEX()));
    pc += 2;
    break;

    case 0x19:
  	ORA(MEM(ABSOLUTEY()));
    pc += 2;
    break;

    case 0x01:
  	ORA(MEM(INDIRECTX()));
    pc++;
    break;

    case 0x11:
  	ORA(MEM(INDIRECTY()));
    pc++;
    break;

    case 0x48:
    PUSH(a);
    break;

    case 0x08:
    PUSH(flags);
    break;

    case 0x68:
    ASSIGNSETFLAGS(a, POP());
    break;

    case 0x28:
    flags = POP();
    break;

    case 0x2a:
    ROL(a);
    break;

    case 0x26:
    ROL(MEM(ZEROPAGE()));
    WRITE(ZEROPAGE());
    pc++;
    break;

    case 0x36:
    ROL(MEM(ZEROPAGEX()));
    WRITE(ZEROPAGEX());
    pc++;
    break;

    case 0x2e:
    ROL(MEM(ABSOLUTE()));
    WRITE(ABSOLUTE());
    pc += 2;
    break;

    case 0x3e:
    ROL(MEM(ABSOLUTEX()));
    WRITE(ABSOLUTEX());
    pc += 2;
    break;

    case 0x6a:
    ROR(a);
    break;

    case 0x66:
    ROR(MEM(ZEROPAGE()));
    WRITE(ZEROPAGE());
    pc++;
    break;

    case 0x76:
    ROR(MEM(ZEROPAGEX()));
    WRITE(ZEROPAGEX());
    pc++;
    break;

    case 0x6e:
    ROR(MEM(ABSOLUTE()));
    WRITE(ABSOLUTE());
    pc += 2;
    break;

    case 0x7e:
    ROR(MEM(ABSOLUTEX()));
    WRITE(ABSOLUTEX());
    pc += 2;
    break;

    case 0x40:
    if (sp == 0xff) return 1;
    flags = POP();
    pc = POP();
    pc |= POP() << 8;
    break;

    case 0x60:
    if (sp == 0xff) return 1;
    pc = POP();
    pc |= POP() << 8;
    pc++;
    break;

  	case 0xe9:
  	SBC(IMMEDIATE());
    pc++;
  	break;

  	case 0xe5:
  	SBC(MEM(ZEROPAGE()));
    pc++;
  	break;

  	case 0xf5:
  	SBC(MEM(ZEROPAGEX()));
  	pc++;
    break;

    case 0xed:
    SBC(MEM(ABSOLUTE()));
    pc += 2;
    break;

    case 0xfd:
    SBC(MEM(ABSOLUTEX()));
    pc += 2;
    break;

    case 0xf9:
    SBC(MEM(ABSOLUTEY()));
    pc += 2;
    break;

    case 0xe1:
  	SBC(MEM(INDIRECTX()));
    pc++;
    break;

    case 0xf1:
  	SBC(MEM(INDIRECTY()));
  	pc++;
    break;

    case 0x38:
    flags |= FC;
    break;

    case 0xf8:
    flags |= FD;
    break;

    case 0x78:
    flags |= FI;
    break;

  	case 0x85:
  	MEM(ZEROPAGE()) = a;
  	WRITE(ZEROPAGE());
    pc++;
  	break;

  	case 0x95:
  	MEM(ZEROPAGEX()) = a;
  	WRITE(ZEROPAGEX());
    pc++;
    break;

    case 0x8d:
  	MEM(ABSOLUTE()) = a;
  	WRITE(ABSOLUTE());
    pc += 2;
    break;

    case 0x9d:
  	MEM(ABSOLUTEX()) = a;
  	WRITE(ABSOLUTEX());
    pc += 2;
    break;

    case 0x99:
  	MEM(ABSOLUTEY()) = a;
  	WRITE(ABSOLUTEY());
    pc += 2;
    break;

    case 0x81:
  	MEM(INDIRECTX()) = a;
  	WRITE(INDIRECTX());
    pc++;
    break;

    case 0x91:
  	MEM(INDIRECTY()) = a;
  	WRITE(INDIRECTY());
    pc++;
    break;

    case 0x86:
  	MEM(ZEROPAGE()) = x;
  	WRITE(ZEROPAGE());
    pc++;
    break;

    case 0x96:
  	MEM(ZEROPAGEY()) = x;
  	WRITE(ZEROPAGEY());
    pc++;
    break;

    case 0x8e:
  	MEM(ABSOLUTE()) = x;
  	WRITE(ABSOLUTE());
    pc += 2;
    break;

    case 0x84:
  	MEM(ZEROPAGE()) = y;
  	WRITE(ZEROPAGE());
    pc++;
    break;

    case 0x94:
  	MEM(ZEROPAGEX()) = y;
  	WRITE(ZEROPAGEX());
    pc++;
    break;

    case 0x8c:
  	MEM(ABSOLUTE()) = y;
  	WRITE(ABSOLUTE());
    pc += 2;
    break;

    case 0xaa:
    ASSIGNSETFLAGS(x, a);
    break;

    case 0xba:
    ASSIGNSETFLAGS(x, sp);
    break;

    case 0x8a:
    ASSIGNSETFLAGS(a, x);
    break;

    case 0x9a:
    ASSIGNSETFLAGS(sp, x);
    break;

    case 0x98:
    ASSIGNSETFLAGS(a, y);
    break;

    case 0xa8:
    ASSIGNSETFLAGS(y, a);
    break;

    // BRK instruction - API call dispatcher
    case 0x00:
    {
        u8 call = FETCH();
        switch(call)
        {
            case CALL_DEBUGHALT:
            setvblankroutine(0);
            setplayroutine(0);
            while (1)
            {

                REG_DISPCNT = DISPCNT_VIDEOMODE(0) | DISPCNT_1DSPRITES;
                PALRAM[0]++;
            }
            break;

            case CALL_REDRAW:
            redraw();
            break;

            case CALL_SETSCREENBASE:
            setscreenbase(a);
            break;

            case CALL_SETCOLORBASE:
            setcolorbase(a);
            break;

            case CALL_SETCHARBASE:
            setcharbase(a);
            break;
            
            case CALL_SETSPRITEBASE:
            setspritebase(a);
            break;

            case CALL_SETSIDBASE:
            setsidbase(a);
            break;

            case CALL_SETGAMECOLORS:
            setgamecolors(a,x,y);
            break;

            case CALL_SETSCORECOLORS:
            setscorecolors(a,x,y);
            break;

            case CALL_SETGAMESCROLL:
            setgamescroll(x,y);
            break;

            case CALL_PRINTOVERLAY:
            printoverlay(a, x);
            break;

            case CALL_SETSPRITECOLORS:
            setspritecolors(x,y);
            break;

            case CALL_SETSPRITELIST:
            setspritelist(y, a | (x << 8));
            break;

            case CALL_GETCONTROLS:
            {
                u16 controls = REG_KEY ^ 0x3ff;

                a = 0;
                x = 0;
                if (controls & KEY_UP) a |= JOY_UP;
                if (controls & KEY_DOWN) a |= JOY_DOWN;
                if (controls & KEY_LEFT) a |= JOY_LEFT;
                if (controls & KEY_RIGHT) a |= JOY_RIGHT;
                if (controls & KEY_A) a |= JOY_FIRE1;
                if (controls & KEY_B) a |= JOY_FIRE2;
                if (controls & KEY_L) x |= JOY_L;
                if (controls & KEY_R) x |= JOY_R;
                if (controls & KEY_SELECT) x |= JOY_SELECT;
                if (controls & KEY_START) x |= JOY_START;
            }
            break;

            case CALL_SETBLITTERSRC:
            blittersrc = a | (x << 8);
            break;

            case CALL_SETBLITTERDEST:
            blitterdest = a | (x << 8);
            break;

            case CALL_RUNBLITTER:
            if (runblitter(blitterdest, blittersrc, a | (x << 8)))
                flags &= ~FC;
            else
                flags |= FC;
            break;

            case CALL_SETVBLANKROUTINE:
            setvblankroutine(a | (x << 8));
            break;
            
            case CALL_SETPLAYROUTINE:
            setplayroutine(a | (x << 8));
            break;
            
            case CALL_OPENFILE:
            filename[0] = tolower(a);
            filename[1] = tolower(x);
            if (openfile(filename))
                flags &= ~FC;
            else
                flags |= FC;
            break;

            case CALL_GETBYTE:
            if (fileend())
            {
                a = 0;
                flags |= FC;
            }
            else
            {
                a = getbyte();
                flags &= ~FC;
            }
            break;
            
            case CALL_LOADSUBFILE:
            if (loadsubfile(a | (x << 8)))
                flags &= ~FC;
            else
                flags |= FC;
            break;

            case CALL_SETSAVESTATESTART:
            setsavestatestart(a | (x << 8));
            break;

            case CALL_SETSAVESTATESIZE:
            setsavestatesize(a | (x << 8));
            break;

            case CALL_LOADSTATE:
            if (loadstate(a))
                flags &= ~FC;
            else
                flags |= FC;
            break;
            
            case CALL_SAVESTATE:
            if (savestate(a))
                flags &= ~FC;
            else
                flags |= FC;
            break;
            
            case CALL_RANDOM:
            a = getrandom();
            break;
        }
    }
    break;

    default:
    return 0;
  }

  }
}

CODE_IN_IWRAM void writerange(u16 start, u16 end)
{
    int c;

    for (c = (start >> 6); c <= (end >> 6); c++)
        cpuwritemap[c] = 1;
}

CODE_IN_IWRAM void clearrange(u16 start, u16 end)
{
    int c;

    for (c = (start >> 6); c <= (end >> 6); c++)
        cpuwritemap[c] = 0;
}
