;
; MW4 Scriptfile 01: Title, game menu, some common things
;

                include scriptm.s               ;Script macros etc.

                entrypoint titleinit            ;100 - Titlescreen init
                entrypoint titlerun             ;101 - Titlescreen running
                entrypoint statusscreen         ;102 - Status screen
                entrypoint gamemenu             ;103 - Game menu
                entrypoint surgicalstation      ;104 - Surgical unit
                entrypoint recharger            ;105 - Battery recharger
                entrypoint useitems             ;106 - Use player items
                entrypoint newgame              ;107 - New game
                entrypoint finalsurgical1       ;108 - Final level s.unit 1
                entrypoint finalrecharger1      ;109 - Final level rech. 1
                entrypoint finalsurgical2       ;10a - Final level s.unit 2
                entrypoint finalrecharger2      ;10b - Final level rech. 2

;-------------------------------------------------------------------------------
; $0100 - Title init
;-------------------------------------------------------------------------------

titleinit:      jsr resetmessage
                jsr initactors
                jsr cleartextscreen
                lda #$7f                        ;Set energybars to be redrawn
                sta plrprevhp
                sta plrprevbatt
                lda #$01                        ;Play titletune
                jsr playgametune
                lda #$00
                sta musiccntlo
                sta musiccnthi
                sta gameon
                sta titlereveal
                sta cheatindex
                ldx #FILE_TITLE                 ;Load title logo / texts
                jsr makefilename
                lda #<map
                ldx #>map
                jsr loadfileretry
                lda #$60                        ;Reset BG anim
                sta bganimcode
                ldx #$00                        ;Copy title charset
ccs_loop:       lda map,x
                sta chars+$200,x
                lda map+$100,x
                sta chars+$300,x
                lda map+$200,x
                sta chars+$400,x
                inx
                bne ccs_loop
ti_logo:        lda map+$2b0,x               ;Draw logo
                sta textscreen+1*40,x
                lda map+$2b0+220,x
                sta textscreen+1*40+220,x
                inx
                cpx #220
                bcc ti_logo
                lda #<SCRIPT_TITLERUN        ;Set title runloop
                ldx #>SCRIPT_TITLERUN
                jsr execlatentscript
ti_resetptr:    lda #<(map+$470)             ;Reset title textptr.
                sta titleptrlo
                lda #>(map+$470)
                sta titleptrhi
unused:         rts

;-------------------------------------------------------------------------------
; $0101 - Title runloop
;-------------------------------------------------------------------------------

titlerun:       lda keytype
                bmi tr_nocheat

                ldy cheatindex                  ;Check cheatstring
                bne tr_cheatnotfirst            ;The 2 cheatstrings are stored
tr_cheatbegin:  cmp cheatstring,y               ;in interleaved fashion
                beq tr_cheatfound
                iny
tr_cheatnotfirst:
                cmp cheatstring,y
                bne tr_cheatwrong
tr_cheatfound:  iny                             ;Advance by 2 to go to the
                iny                             ;next letter of the string
                lda cheatstring,y               ;Check for endmark
                bpl tr_cheatcontinue
                lsr
                bcc tr_lifecheat
tr_ammocheat:   lda ah_limitedammo+1
                eor #$01
                sta ah_limitedammo+1
                bcs tr_cheatdonecommon

tr_lifecheat:   lda actl_agent+AD_DEFAULTHP
                eor #%11001110                  ;Toggle invulnerability
                sta actl_agent+AD_DEFAULTHP
tr_cheatdonecommon:
                dec rgscr_textbg1+1
                jsr redraw
                inc rgscr_textbg1+1

tr_cheatwrong:  ldy #$00
tr_cheatcontinue:
                sty cheatindex
tr_nocheat:     ldy titlereveal                 ;Reveal logo by writing to
                cpy #30                         ;color-RAM
                beq tr_wait
                lda #$09
                sta colors+1*40,y
                sta colors+2*40,y
                sta colors+3*40,y
                sta colors+4*40,y
                sta colors+5*40,y
                sta colors+6*40,y
                sta colors+7*40,y
                sta colors+8*40,y
                sta colors+9*40,y
                sta colors+10*40,y
                sta colors+11*40,y
                lda #$01
                sta colors+13*40,y
                sta colors+14*40,y
                sta colors+15*40,y
                sta colors+16*40,y
                jsr tr_setptr
                lda (temp1),y                   ;Print lower part texts
                and #$3f
                sta textscreen+13*40,y
                jsr tr_add40
                sta textscreen+14*40,y
                jsr tr_add40
                sta textscreen+15*40,y
                jsr tr_add40
                sta textscreen+16*40,y
                inc titlereveal
tr_waitdone:    rts
tr_wait:        lda musiccnthi
                cmp #$02
                bcc tr_waitdone
                lda #$00
                sta musiccntlo
                sta musiccnthi
                sta titlereveal
                lda titleptrlo
                clc
                adc #160
                sta titleptrlo
                bcc tr_notover
                inc titleptrhi
tr_notover:     jsr tr_setptr
                ldy #$00
                lda (temp1),y
                bpl tr_waitdone
                jmp ti_resetptr

tr_setptr:      lda titleptrlo
                sta temp1
                lda titleptrhi
                sta temp2
                rts

tr_add40:       lda temp1
                clc
                adc #40
                sta temp1
                bcc tr_add40ok
                inc temp2
tr_add40ok:     lda (temp1),y
                and #$3f
                rts

titleptrlo:     dc.b 0
titleptrhi:     dc.b 0
titlereveal:    dc.b 0
titledelay:     dc.b 0

cheatindex:     dc.b 0

cheatstring:    dc.b KEY_L,KEY_R        ;LRLR,SEL,START = unlimited reloads
                dc.b KEY_R,KEY_L        ;RLRL,START,SEL = unlimited life
                dc.b KEY_L,KEY_R
                dc.b KEY_R,KEY_L
                dc.b KEY_SELECT,KEY_START
                dc.b KEY_START,KEY_SELECT
                dc.b $81,$80

;-------------------------------------------------------------------------------
; $0102 - Status screen
;-------------------------------------------------------------------------------

statusscreen:   jsr resetmessage
                lda #<statustext1
                ldy #>statustext1
                jsr menu_titletext
                lda carrylo                     ;Rotate highest bit
                rol                             ;to bit 0 for rounding
                rol                             ;weight upwards
                and #$01
                clc
                adc carryhi
                ldx #14
                jsr menu_printnumber
                lda carrymax                    ;Print carrying max. amount
                ldx #17
                jsr menu_printnumber
                lda ti_hours                    ;Print game time
                ldx #22
                jsr menu_printnumber
                lda ti_minutes
                ldx #25
                jsr menu_printnumber
                lda ti_seconds
                ldx #28
                jsr menu_printnumber
                lda #3
                sta dhb_nohalf+1
                tay
                lda #$08
menu_barcolorloop:
                sta colors+22*40+10,y
                sta colors+22*40+17,y
                sta colors+22*40+26,y
                dey
                bne menu_barcolorloop
                lda plrvit
                clc
                adc #$01
                ldx #51
                jsr dhb_noadjust
                lda plrstr
                clc
                adc #$01
                ldx #58
                jsr dhb_noadjust
                lda plrarmor
                ldx #67
                jmp dhb_noadjust

;-------------------------------------------------------------------------------
; $0103 - Game menu
;-------------------------------------------------------------------------------

gamemenu:       lsr
                bcs gamemenuchoice

gamemenuredraw: jsr resetmessage
                lda #<gamemenutext1             ;Redraw menu
                ldy #>gamemenutext1
                jsr menu_titletext
                ldy gmselect
                ldx gamemenucursortbl,y
                lda #37
                sta textscreen+21*40,x
                ldx #29
                lda musicmode
                jsr menu_onofftext
                ldx #69
                lda soundmode
                jmp menu_onofftext

gamemenuchoice: ldy gmselect                    ;Perform menu selection
                beq gm_new
                dey
                beq gm_load
                dey
                beq gm_save
                dey
                beq gm_exit
                dey
                beq gm_music
gm_sound:       lda soundmode                   ;Toggle sound effects
                eor #$01
                sta soundmode
                jmp menu_redraw

gm_music:       lda musicmode                   ;Toggle music
                eor #$01
                sta musicmode
                jsr togglemusic
                jmp menu_redraw

gm_new:         jsr resetmessage                ;Start new game: select
                jmp gm_newredraw

gm_noexit:      jmp menu_redraw
gm_exit:        lda gameon
                beq gm_noexit
                jsr resetmessage
                jmp gm_exitredraw

gm_savefail:    jmp menu_redraw
gm_load:        lda #$00
                beq gm_savecommon
gm_save:        lda gameon                      ;Only allow save in game
                cmp #STATUS_INGAME              ;(not title or gameover)
                bne gm_savefail
                lda #$01
gm_savecommon:  sta gm_savemode+1
                jsr resetmessage
gm_saveredraw:  lda #SFX_SELECT
                jsr playsfx
                lda #<savemenutext1
                ldy #>savemenutext1
                ldx gm_savemode+1
                bne gm_savemsgok
                lda #<loadmenutext1
                ldy #>loadmenutext1
gm_savemsgok:   ldx #8
                jsr printpaneltext
                lda #<savemenutext2
                ldy #>savemenutext2
                ldx #48
                jsr printpaneltext
                ldy sgselect
                ldx savemenucursortbl,y
                lda #37
                sta textscreen+21*40,x
gm_saveloop:    jsr menu_control
                lda temp1
                bmi gm_savegoback
                and #JOY_FIRE|JOY_JUMP
                bne gm_saveselect
gm_savenoselect:lda temp1
                lsr
                bcs gm_saveleft
                lsr
                bcs gm_saveright
                lsr
                bcs gm_saveleft
                lsr
                bcs gm_saveright
                jmp gm_saveloop
gm_savegoback:  jsr menu_exitwithoutsound
                jmp menu_redraw
gm_saveleft:    dec sgselect
                bpl gm_saveredraw
                lda #NUMGAMES
                sta sgselect
                bne gm_saveredraw
gm_saveright:   inc sgselect
                lda sgselect
                cmp #NUMGAMES+1
                bcc gm_saveredraw
                lda #$00
                sta sgselect
                beq gm_saveredraw
gm_saveselect:  lda sgselect
                cmp #NUMGAMES                   ;Cancel
                bcs gm_savegoback
gm_savemode:    ldy #$00
                beq gm_doload
gm_dosave:      cmp #GAME_AUTOSAVE              ;Do not write manually over
                beq gm_saveloop                 ;autosave
                jsr showdisk
                callapi C_SAVESTATE
                ldx #MENU_SAVEDELAY
gm_savedelay:   callapi C_REDRAW                ;Delay so that user
                dex                             ;believes it saved :)
                bne gm_savedelay
gm_failload:    jsr hidedisk
                jmp menu_continuegame
gm_doload:      jsr showdisk                    ;Try to load savefile
                callapi C_LOADSTATE
                bcs gm_failload
gm_loaddone:    jsr cleartextscreen             ;Do not show flashing etc.
                jsr hidedisk                    ;while zone colors change
                lda #UPD_ITEM|UPD_AMMO|UPD_SCORE  ;Update all
                sta panelupdateflag
                ldx #$7f                          ;Backup zone colors
gm_backupcols:  lda zonebg1,x
                sta colorbackup,x
                dex
                bpl gm_backupcols
                lda levelnum                    ;Disable autosave when loading game
                sta autosavelevelnum            ;(save at next level change)
                ldx #LLMODE_NOPURGEABLE
                jsr loadlevel
                ldx #$7f
gm_restorecols: lda colorbackup,x               ;Restore zone colors now
                sta zonebg1,x
                dex
                bpl gm_restorecols
                lda actt+ACTI_PLAYER            ;If not an Agent yet, the cheat
                cmp #ACT_IANAGENT               ;is insignificant
                bne gm_nocheat
                lda acthp+ACTI_PLAYER           ;If cheat switched off,
                cmp actl_agent+AD_DEFAULTHP     ;reflect this
                bcc gm_nohealthreset
                lda actl_agent+AD_DEFAULTHP
                sta acthp+ACTI_PLAYER
gm_nohealthreset:                               ;If cheat switched on,
                lda actl_agent+AD_DEFAULTHP     ;reflect also this
                bpl gm_nocheat
                sta acthp+ACTI_PLAYER
gm_nocheat:     jsr setplayerstatus             ;Set correct damage subtract
                jsr resetmessage
                lda tunenum                     ;Restart the tune that should
                jsr playgametune                ;be playing
                jmp centerplayer                ;Go back to game

gm_newredraw:   lda #SFX_SELECT                 ;difficulty
                jsr playsfx
                lda #<diffmenutext1
                ldy #>diffmenutext1
                jsr menu_titletext
                ldy dfselect
                ldx diffmenucursortbl,y
                lda #37
                sta textscreen+21*40,x
gm_newloop:     jsr menu_control
                lda temp1
                bmi gm_newgoback
                and #JOY_FIRE|JOY_JUMP
                bne gm_newselect
gm_newnoselect: lda temp1
                lsr
                bcs gm_newleft
                lsr
                bcs gm_newright
                lsr
                bcs gm_newleft
                lsr
                bcs gm_newright
                jmp gm_newloop
gm_newleft:     dec dfselect
                jmp gm_newmovecommon
gm_newright:    inc dfselect
gm_newmovecommon:
                lda dfselect
                and #$03
                sta dfselect
                bpl gm_newredraw
gm_newgoback:   jsr menu_exitwithoutsound
                jmp menu_redraw
gm_newselect:   lda dfselect
                cmp #$03
                bcs gm_newgoback
                sta difficulty                  ;Store difficulty selection
                lda #<SCRIPT_NEWGAME           ;Game starts
                ldx #>SCRIPT_NEWGAME
                jmp execscript

gm_exitredraw:  lda #SFX_SELECT
                jsr playsfx
                lda #<exitmenutext1
                ldy #>exitmenutext1
                jsr menu_titletext
                ldy qtselect
                ldx exitmenucursortbl,y
                lda #37
                sta textscreen+21*40,x
gm_exitloop:    jsr menu_control
                lda temp1
                bmi gm_exitgoback
                and #JOY_FIRE|JOY_JUMP
                bne gm_exitselect
gm_exitnoselect:lda temp1
                lsr
                bcs gm_exitleft
                lsr
                bcs gm_exitright
                lsr
                bcs gm_exitleft
                lsr
                bcs gm_exitright
                jmp gm_exitloop
gm_exitleft:    dec qtselect
                bpl gm_exitredraw
                lda #$01
                sta qtselect
                bne gm_exitredraw
gm_exitright:   inc qtselect
                lda qtselect
                cmp #$02
                bcc gm_exitredraw
                lda #$00
                sta qtselect
                beq gm_exitredraw
gm_exitgoback:  jsr menu_exitwithoutsound
                jmp menu_redraw
gm_exitselect:  lda qtselect
                bne gm_exitgoback
gm_endgame:     lda #$00                        ;Reset choice to "new game"
                sta gmselect
                lda #<SCRIPT_TITLE             ;Go back to title
                ldx #>SCRIPT_TITLE
                jmp execscript

menu_printnumber:
                sta temp3                       ;Subroutine to print a 2 digit
                stx maptemp1                    ;number
                jsr convert8bits
                lda temp4
                ldx maptemp1
                jmp printbcddigits

menu_onofftext: cmp #$01                        ;Print either 'on' or 'off'
                lda #<offtext
                ldy #>offtext
                bcc menu_onofftext2
                lda #<ontext
                ldy #>ontext
menu_onofftext2:jmp printpaneltext

menu_titletext: ldx #8
                jsr printpaneltext
                ldx #48
                jmp printpaneltextcont

;-------------------------------------------------------------------------------
; $0104 - Surgical station
;-------------------------------------------------------------------------------

surgicalstation:jsr menu_inactivatecounter
                ldx #ACTI_PLAYER
                jsr mh_setdoorframe
                lda #3
                sta ss_select
ss_redraw2:     jsr resetmessage
ss_redraw:      lda #<ss_text
                ldy #>ss_text
                jsr menu_titletext
                lda #SFX_SELECT
                jsr playsfx
                ldy ss_select
                ldx ss_cursortbl,y
                lda #37
                sta textscreen+21*40,x
ss_loop:        jsr increaseclock
                jsr menu_control
                lda temp1
                bmi ss_exit
                and #JOY_FIRE|JOY_JUMP
                bne ss_choose
                lda temp1
                lsr
                bcs ss_left
                lsr
                bcs ss_right
                lsr
                bcs ss_left
                lsr
                bcs ss_right
                jmp ss_loop
ss_choose:      ldx #ACTI_PLAYER
                lda ss_select
                choice 0,ss_heal
                choice 1,ss_vit
                choice 2,ss_str
ss_exit:        jsr mh_setstandingframe
                jmp menu_exit

ss_left:        lda #$ff
                bne ss_moveok
ss_right:       lda #$01
ss_moveok:      clc
                adc ss_select
                and #$03
                sta ss_select
ssh_notok:      jmp ss_redraw

ss_heal:        lda acthp,x
                cmp #HP_AGENT
                bcs ssh_notok
                lda #(HP_AGENT)/12             ;Restore full hitpoints
                sta acthpdelta,x
                lda #12
                sta acthptime,x
                lda #SFX_HYPO
                jsr playsfx
                jmp ss_exit

ss_vit:         lda #SFX_SELECT
                jsr playsfx
                lda #ITEM_VITALITY_ENHANCEMENT
                jsr finditem
                bcc ss_noenhancement
                inc plrvit
ss_enhcommon:   jsr setplayerstatus
                lda #SFX_HYPO                   ;Sound & remove from
                jsr useitems_common             ;inventory
ss_successful:  lda #<ss_successtext
                ldx #>ss_successtext
ss_msgcommon:   ldy #MSGTIME_ETERNAL
                jsr printmsgax
                jsr waitformessage
                jmp ss_redraw
ss_noenhancement:
                lda #<ss_notext
                ldx #>ss_notext
                bne ss_msgcommon

ss_str:         lda #SFX_SELECT
                jsr playsfx
                lda #ITEM_STRENGTH_ENHANCEMENT
                jsr finditem
                bcc ss_noenhancement
                inc plrstr
                jmp ss_enhcommon


ss_text:        dc.b "SELECT PROCEDURE:",0
                dc.b " HEAL  VIT  STR  EXIT",0
ss_successtext: dc.b "UPGRADE INSTALLED",0
ss_notext:      dc.b "NO UPGRADE AVAILABLE",0

ss_cursortbl:   dc.b 48,54,59,64

ss_select:      dc.b 0

;-------------------------------------------------------------------------------
; $0105 - Energy recharger
;-------------------------------------------------------------------------------

recharger:      ldx #ACTI_PLAYER
                lda plrarmor                    ;Has armor at all?
                beq rc_notok
                lda actbatt,x
                cmp #HP_AGENT
                bcs rc_notok
                lda #HP_AGENT/24
                sta plrbattdelta
                lda #24
                sta plrbatttime
                jsr mh_setdoorframe
                lda #SFX_POWERUP
                jsr playsfx
                lda #<recharger_text
                ldx #>recharger_text
                ldy #MSGTIME
                jmp printmsgax
rc_notok:       rts

recharger_text: dc.b "ARMOR RECHARGING..",0

;-------------------------------------------------------------------------------
; $0106 - Use player items
;-------------------------------------------------------------------------------

useitems:       ldy invselect
                lda invtype,y
                choice ITEM_MEDIKIT, usemedikit
                choice ITEM_BATTERY, usebattery
                choice ITEM_BEER, usebeer
                choice ITEM_AGENT_GEAR, usegear
                choice ITEM_LETTER, useletter
                choice ITEM_LEVITATION_DEVICE, uselevitation
                cmp #ITEM_BETA_ARMORSYSTEM
                bcc ui_noarmor
                cmp #ITEM_EPSILON_ARMORSYSTEM+1
                bcs ui_noarmor
                jmp usearmor
ua_skipmsg
ua_notbetter:
ui_noarmor:     rts

useletter:      jumpto SCRIPT_LETTER

usearmor:       sec
                sbc #ITEM_BETA_ARMORSYSTEM-2
                cmp plrarmor                    ;Must be better or equal as
                bcc ua_notbetter                ;previous
usegear2:       sta plrarmor
                jsr setplayerstatus
                lda #0                          ;New armor starts from zero
                sta actbatt+ACTI_PLAYER
                lda #HP_AGENT/24
                sta plrbattdelta                ;But charges to maximum
                lda #24
                sta plrbatttime
                jsr usepowerup_common           ;Sound + decrease ammo
                lda plrarmor                    ;First Agent gear?
                cmp #$01
                beq ua_skipmsg
                lda #<armortext
                ldx #>armortext
                ldy #MSGTIME
                jmp printmsgax

usemedikit:     lda acthp+ACTI_PLAYER
                cmp #HP_AGENT
                bcs ub_notok
                lda acthptime+ACTI_PLAYER       ;Do not interrupt previous
                beq umk_ok                      ;healing
                bmi umk_ok
usel_skip:
ub_notok:       rts
umk_ok:         lda #(HP_AGENT/2)/6            ;Restore half of hitpoints
                sta acthpdelta+ACTI_PLAYER
                lda #6
                sta acthptime+ACTI_PLAYER
                lda #SFX_HYPO
                jmp useitems_common

usebeer:        lda #(HP_AGENT/3)/4             ;Beer restores 1/3 hitpoints
                sta acthpdelta+ACTI_PLAYER      ;and can be consumed any time :)
                lda #4
                sta acthptime+ACTI_PLAYER
                bne usepowerup_common

usegear:        lda #$01
                jsr usegear2                    ;Exec certain script when
                lda #<SCRIPT_SARGECONV2         ;wearing the Agent gear
                ldx #>SCRIPT_SARGECONV2         ;(in latent mode)
                jmp execlatentscript

usebattery:     lda plrarmor                    ;Has armor at all?
                beq ub_notok
                lda actbatt+ACTI_PLAYER
                cmp #HP_AGENT
                bcs ub_notok
                lda plrbatttime                 ;Do not interrupt previous
                beq ub_ok                       ;power-up
                lda plrbattdelta
                bpl ub_notok
ub_ok:          lda #(HP_AGENT/2)/12
                sta plrbattdelta
                lda #12
                sta plrbatttime
usepowerup_common:
                lda #SFX_POWERUP
useitems_common:jsr playsfx
                lda #$01                        ;Decrement item amount by one
                jmp decreaseammo

uselevitation:  lda actt+ACTI_PLAYER            ;Traitors can't use the levitation
                cmp #ACT_IANAGENT               ;device anymore :)
                bne usel_skip
                lda #PLOT_LDEVICE_ACTIVE
                jsr checkplotbit
                beq usel_activate
usel_deactivate:lda #PLOT_LDEVICE_ACTIVE
                jsr clearplotbit
                jsr setplayerstatus
                lda #ITEM_LEVITATION_DEVICE
                jsr getitemname
                ldy #MSGTIME
                jsr printmsgptr
                lda #MSG_OFF
                jsr getmsgptr
                jsr ptw_continue
                lda #SFX_SHUTDOWN
                jmp playsfx

usel_activate:  lda #PLOT_LDEVICE_ACTIVE
                jsr setplotbit
                jsr setplayerstatus
                lda #ITEM_LEVITATION_DEVICE
                jsr getitemname
                ldy #MSGTIME
                jsr printmsgptr
                lda #MSG_ON
                jsr getmsgptr
                jsr ptw_continue
                lda #SFX_TRAIN
                jmp playsfx

armortext:      dc.b "ARMORSYSTEM INSTALLED",0

;-------------------------------------------------------------------------------
; $0107 - New game
;-------------------------------------------------------------------------------

START_LEVEL     = $02                           ;Proper game start
START_X         = $01
START_Y         = $05
START_SCRIPT    = SCRIPT_INTROFADE
START_ACT       = ACT_IANCIVILIAN

;START_LEVEL     = $05                           ;Start from the Campus
;START_X         = $01
;START_Y         = $05
;START_SCRIPT    = SCRIPT_INTROFADE
;START_ACT       = ACT_IANAGENT

;START_LEVEL     = $06                           ;Start from Comm.Interception
;START_X         = $05
;START_Y         = $1d
;START_SCRIPT    = SCRIPT_INTROFADE
;START_ACT       = ACT_IANAGENT

;START_LEVEL     = $08                           ;Start from Erehwon
;START_X         = $07
;START_Y         = $0d
;START_SCRIPT    = SCRIPT_INTROFADE
;START_ACT       = ACT_IANAGENT

;START_LEVEL     = $0a                           ;Start from Reseach outside
;START_X         = $39
;START_Y         = $05
;START_SCRIPT    = SCRIPT_INTROFADE
;START_ACT       = ACT_IANAGENT

;START_LEVEL     = $0b                           ;Start from Reseach bossfight
;START_X         = $27
;START_Y         = $3d
;START_SCRIPT    = SCRIPT_INTROFADE
;START_ACT       = ACT_IANAGENT

;START_LEVEL     = $06                           ;Start next to the code in
;START_X         = $7a                           ;CommInt
;START_Y         = $05
;START_SCRIPT    = SCRIPT_INTROFADE
;START_ACT       = ACT_IANAGENT

;START_LEVEL     = $03                           ;Start from Agency, perform
;START_X         = $22                          ;the decoding right away
;START_Y         = $0f
;START_SCRIPT    = SCRIPT_SETDECODETRIGGER
;START_ACT       = ACT_IANAGENT

;START_LEVEL     = $0e                          ;Start from outside mansion
;START_X         = $00
;START_Y         = $31
;START_SCRIPT    = SCRIPT_INTROFADE
;START_ACT       = ACT_IANAGENT

;START_LEVEL     = $0f                          ;Start from mansion
;START_X         = $04
;START_Y         = $0c
;START_SCRIPT    = SCRIPT_INTROFADE
;START_ACT       = ACT_IANAGENT

;START_LEVEL     = $0f                          ;Start from Suhrim bossfight
;START_X         = $26
;START_Y         = $1a
;START_SCRIPT    = SCRIPT_INTROFADE
;START_ACT       = ACT_IANAGENT

;START_LEVEL     = $11                          ;Start from Global Influence
;START_X         = $24
;START_Y         = $05
;START_SCRIPT    = SCRIPT_INTROFADE
;START_ACT       = ACT_IANAGENT

;START_LEVEL     = $12                          ;Start from Inner Sphere Hub
;START_X         = $3c
;START_Y         = $0f
;START_SCRIPT    = SCRIPT_INTROFADE
;START_ACT       = ACT_IANAGENT

;START_LEVEL     = $12                          ;Start from Exoskeleton bossfight
;START_X         = $89                          ;(teleport)
;START_Y         = $0a
;START_SCRIPT    = SCRIPT_INTROFADE
;START_ACT       = ACT_IANAGENT

;START_LEVEL     = $19                          ;Start from the final level
;START_X         = $02
;START_Y         = $05
;START_SCRIPT    = SCRIPT_INTROFADE
;START_ACT       = ACT_IANAGENT

;START_LEVEL     = $19                          ;Start from Ahriman bossfight
;START_X         = $4c
;START_Y         = $11
;START_SCRIPT    = SCRIPT_INTROFADE
;START_ACT       = ACT_IANAGENT

;START_LEVEL     = $19                          ;Start from Lilith bossfight
;START_X         = $59
;START_Y         = $23
;START_SCRIPT    = SCRIPT_INTROFADE
;START_ACT       = ACT_IANAGENT

;START_LEVEL     = $04                          ;Start from Goatforest
;START_X         = $ae
;START_Y         = $0e
;START_SCRIPT    = SCRIPT_INTROFADE
;START_ACT       = ACT_IANAGENT

;START_LEVEL     = $17                          ;Start from Sentinel Temple
;START_X         = $38
;START_Y         = $16
;START_SCRIPT    = SCRIPT_INTROFADE
;START_ACT       = ACT_IANAGENT

;START_LEVEL     = $17                          ;Start from exiting Sentinel mothership
;START_X         = $38
;START_Y         = $16
;START_SCRIPT    = SCRIPT_SENTINELREUNIONEND
;START_ACT       = ACT_IANAGENT

;START_LEVEL     = $17                          ;Start from endsequence
;START_X         = $38
;START_Y         = $16
;START_SCRIPT    = SCRIPT_ENDSEQUENCE
;START_ACT       = ACT_IANAGENT

newgame:        jsr initplayer                  ;Script-independent init

                ldy #$02
initcode:       jsr random                      ;Random teleport code in
                and #$0f                        ;each game
                cmp #$0a
                bcs initcode
                sta teleportcode,y
                dey
                bpl initcode

                ldx #NUMGRP-1                   ;Initial group alliances
initalliance:   lda initalliancetbl,x
                sta alliance,x
                dex
                bpl initalliance

                ldy #$00
                ldx #$00
inittriggerloop:lda inittriggertbl,x            ;Initial actortrigger list
                beq inittriggerdone
                sta attype,y
                lda inittriggertbl+1,x
                sta atscriptep,y
                lda inittriggertbl+2,x
                sta atscriptf,y
                lda inittriggertbl+3,x
                sta atmask,y
                iny
                txa
                clc
                adc #$04
                tax
                bne inittriggerloop
inittriggerdone:

                ldx #ITEM_CREDITS               ;Add some credts
                lda #20
                jsr additem
                lda #PLOT_MAP_SUBCITY           ;Allowed locations to travel
                jsr setplotbit
                lda #PLOT_MAP_FARM
                jsr setplotbit

                if START_LEVEL = $17
                ldx #ITEM_CRYSTAL
                lda #1
                jsr additem
                endif

                lda #START_LEVEL                ;Load starting level
                sta autosavelevelnum            ;Don't autosave until first level change
                ldx #LLMODE_NORMAL
                jsr loadlevel

                ldy #ACTI_PLAYER                ;Create player actor
                lda #START_ACT
                jsr createactor
                tya
                tax
                lda #START_X                    ;Set starting location
                sta actxh,x
                lda #START_Y
                sta actyh,x
                jsr initcomplexactor
                lda #$00
                sta actd,x                      ;Facing right
                jsr getplayerzone
                jsr playzonetune                ;Start zone music
                jumpto START_SCRIPT             ;Go to starting script

inittriggertbl: dc.b ACT_STREETCOP,<SCRIPT_COP,>SCRIPT_COP,T_TAKEDOWN|T_CONV
                dc.b ACT_WEAPONSHOP,<SCRIPT_WEAPONSHOP,>SCRIPT_WEAPONSHOP,T_CONV
                dc.b ACT_TECHBOSS,<SCRIPT_TECHBOSS,>SCRIPT_TECHBOSS,T_APPEAR|T_TAKEDOWN|T_REMOVE
                dc.b ACT_EXOSKELETON,<SCRIPT_EXOSKELETON,>SCRIPT_EXOSKELETON,T_APPEAR|T_TAKEDOWN|T_REMOVE
                dc.b ACT_MUTANT2,<SCRIPT_MUTANT,>SCRIPT_MUTANT,T_APPEAR|T_TAKEDOWN|T_REMOVE
                dc.b ACT_BARTENDER,<SCRIPT_BARTENDER,>SCRIPT_BARTENDER,T_CONV
                dc.b ACT_BLOWFISH,<SCRIPT_BLOWFISHINTRO,>SCRIPT_BLOWFISHINTRO,T_NEAR|T_CONV
                dc.b ACT_AXESMITH,<SCRIPT_AXESMITH,>SCRIPT_AXESMITH,T_CONV
                dc.b ACT_EXAGENT,<SCRIPT_EXAGENT,>SCRIPT_EXAGENT,T_CONV
                dc.b ACT_SUHRIMBYSTD,<SCRIPT_SUHRIM1,>SCRIPT_SUHRIM1,T_APPEAR
                dc.b ACT_OBSKURIUSHEAD,<SCRIPT_OBSKURIUS,>SCRIPT_OBSKURIUS,T_CONV|T_NEAR
                dc.b ACT_ULTRASHRED,<SCRIPT_DRULTRASHRED,>SCRIPT_DRULTRASHRED,T_CONV
                dc.b ACT_IRONFIST,<SCRIPT_IRONFIST,>SCRIPT_IRONFIST,T_CONV|T_NEAR|T_POSTATTACK
                dc.b ACT_GOAT,<SCRIPT_GOAT,>SCRIPT_GOAT,T_CONV|T_NEAR
                dc.b 0

initalliancetbl:dc.b #%01111111                  ;Bystander hates none
                dc.b #%00001011                  ;Agents hate all except cops
                dc.b #%01000101                  ;SCEPTRE hates all except beasts
                dc.b #%00001111                  ;Cops hate criminals
                dc.b #%00010101                  ;Criminals hate cops, agents
                dc.b #%00100101                  ;and the opposing group but FEAR SCEPTRE!
                dc.b #%01000000                  ;Beasts hate all
                
;-------------------------------------------------------------------------------
; $0108,9,a,b Final level rechargers (dependant on facility power)
;-------------------------------------------------------------------------------

finalsurgical1: getbit PLOT_GENERATOR1_OFFLINE
                bne offline
                jmp surgicalstation
finalrecharger1:getbit PLOT_GENERATOR1_OFFLINE
                bne offline
                jmp recharger
finalsurgical2: getbit PLOT_GENERATOR2_OFFLINE
                bne offline
                jmp surgicalstation
finalrecharger2:getbit PLOT_GENERATOR2_OFFLINE
                bne offline
                jmp recharger

offline:        lda #SFX_OBJECT
                jsr playsfx
                lda #<offlinetext
                ldx #>offlinetext
                ldy #MSGTIME
                jmp printmsgax

offlinetext:    dc.b "OFFLINE",0

                endscript
