;
; MW4 Scriptfile 03: Entrance to SCEPTRE, SCEPTRE train
;
                include scriptm.s               ;Script macros etc.

GPF_LIFT_ID = $01

TRAINSEQUENCE_LENGTH = 96
TRAIN_ACCEL     = $01
TRAIN_MAXSPEED  = $40
TRAIN_SOUND_DELAY = $04

                entrypoint mutant               ;300 - Mutant boss
                entrypoint trainstart           ;301 - SCEPTRE train system
                entrypoint trainrun             ;302 - SCEPTRE train running
                entrypoint liftgpf              ;303 - Malfunctioning lift
                entrypoint liftreset            ;304 - Lift reset button
                entrypoint entersceptre         ;305 - SCEPTRE is entered..
                entrypoint trainplatform        ;306 - Trainstation rant..
                entrypoint unused               ;307 - unused
                entrypoint sceptrecodes         ;308 - SCEPTRE code sheets
                entrypoint techboss             ;309 - Techboss
                entrypoint setdecodetrigger     ;30a - shortcut
                entrypoint iac                  ;30b - IAC project boss

;-------------------------------------------------------------------------------
; $0300 - Mutant
;-------------------------------------------------------------------------------

mutant:         choice T_APPEAR,mutant_appear
                choice T_REMOVE,mutant_remove
mutant_takedown:
                lda #NUMZONES-1
                sta zoneflash
                lda #12
                sta zonebg2+$15
                lda #15
                sta zonebg3+$15
                lda actt,x
                jsr removeactortrigger
mutant_remove:  jmp playzonetune

mutant_appear:  getbit PLOT_SCEPTRE_MUTANT_FIRSTTIME
                jumptrue mutant_skipmsg
                lda #SPRF_MUTANT2   ;Precache the mutant
                jsr loadsprites
                lda #<mutant_text
                ldx #>mutant_text
                ldy #MSGTIME
                jsr printmsgax
                setbit PLOT_SCEPTRE_MUTANT_FIRSTTIME
mutant_skipmsg: lda #$15
                sta zoneflash
                lda #$00
                sta zoneflashcount
                lda tunenum
                and #$fc
                ora #$03
                jmp playgametune

mutant_text:    dc.b "DANGER - EXPERIMENTAL LIFEFORM ESCAPED",0

;-------------------------------------------------------------------------------
; $0301 - SCEPTRE train start
;-------------------------------------------------------------------------------

trainstart:     lda #$00
                sta traincounter
                sta trainanimdelay
                sta trainspeed
                lda sourceobjnum
                and #$01
                sta traindir
                lda #<SCRIPT_TRAINRUN
                ldx #>SCRIPT_TRAINRUN
                jmp execlatentscript

;-------------------------------------------------------------------------------
; $0302 - SCEPTRE train running
;-------------------------------------------------------------------------------

trainrun:       lda trainspeed
                clc
                adc #TRAIN_ACCEL
                cmp #TRAIN_MAXSPEED
                bcc tr_speedok
                lda #TRAIN_MAXSPEED
tr_speedok:     sta trainspeed
                lda trainanimdelay
                clc
                adc trainspeed
                sta trainanimdelay
tr_animloop:    lda trainanimdelay
                cmp #TRAIN_MAXSPEED/2
                bcc tr_noanim
                sbc #TRAIN_MAXSPEED/2
                sta trainanimdelay
                jsr tr_anim
                jmp tr_animloop
tr_noanim:      inc traincounter
                lda traincounter
                cmp #TRAIN_SOUND_DELAY
                bne tr_nosound
                lda #SFX_TRAIN
                jsr playsfx
tr_nosound:     lda traincounter
                cmp #TRAINSEQUENCE_LENGTH
                bcc tr_nostop
                jsr stoplatentscript
                jsr cleartextscreen
                ldy sourceobjnum
                lda tr_desttbl-8,y
                sta ed_destdoor+1
                jmp ed_samelevel
unused:
tr_nostop:      rts

tr_anim:        ldy #$07
                lda traindir
                beq tr_animleft
tr_animright:   lda chars+$f5*8,y
                sta temp1
                lda chars+$f6*8,y
                sta chars+$f5*8,y
                lda chars+$f7*8,y
                sta chars+$f6*8,y
                lda chars+$f8*8,y
                sta chars+$f7*8,y
                lda chars+$f9*8,y
                sta chars+$f8*8,y
                lda chars+$fa*8,y
                sta chars+$f9*8,y
                lda chars+$fb*8,y
                sta chars+$fa*8,y
                lda chars+$fc*8,y
                sta chars+$fb*8,y
                lda temp1
                sta chars+$fc*8,y
                dey
                bpl tr_animright
                rts
tr_animleft:    lda chars+$fc*8,y
                sta temp1
                lda chars+$fb*8,y
                sta chars+$fc*8,y
                lda chars+$fa*8,y
                sta chars+$fb*8,y
                lda chars+$f9*8,y
                sta chars+$fa*8,y
                lda chars+$f8*8,y
                sta chars+$f9*8,y
                lda chars+$f7*8,y
                sta chars+$f8*8,y
                lda chars+$f6*8,y
                sta chars+$f7*8,y
                lda chars+$f5*8,y
                sta chars+$f6*8,y
                lda temp1
                sta chars+$f5*8,y
                dey
                bpl tr_animleft
                rts

tr_desttbl:     dc.b $0e,$0b
                dc.b $08,$0d
                dc.b $0a,$0f
                dc.b $0c,$09


;-------------------------------------------------------------------------------
; $0303 - Lift switch GPF
;-------------------------------------------------------------------------------

liftgpf:        lda #SFX_OBJECT
                jsr playsfx
                checkbit PLOT_SCEPTRE_LIFT_GPF
                bne liftgpf_ok
                lda #<liftgpf_text
                ldx #>liftgpf_text
                ldy #MSGTIME
                jmp printmsgax
liftgpf_ok:     lda #ACT_LIFT1                          ;Activate the lift
                ldx #GPF_LIFT_ID                        ;normally
                jmp ao_aaactorax

liftgpf_text:   dc.b "SWITCH GENERAL PROTECTION FAULT",0

;-------------------------------------------------------------------------------
; $0304 - Lift reset
;-------------------------------------------------------------------------------

liftreset:      checkbit PLOT_SCEPTRE_LIFT_GPF
                bne liftreset_done
                givepoints 5000
                lda #SFX_TRANSMISSION
                jsr playsfx
                lda #PLOT_SCEPTRE_LIFT_GPF
                jmp setplotbit

liftreset_done: lda #SFX_OBJECT
                jmp playsfx


;-------------------------------------------------------------------------------
; $0305 - Enter SCEPTRE
;-------------------------------------------------------------------------------

entersceptre:   givepoints 10000
                setbit PLOT_SCEPTRE_ENTERED
                getbit PLOT_BLACKHAND_DEAD
                jumptrue es_abort
                jsr transmission
                plrsay l_enter1
                saytrans l_enter2
                saytrans l_enter3
es_abort:       stop


l_enter1:       dc.b 34,"I'M AT A TRIANGLE-WITH-EYE SIGN.",34,0
l_enter2:       dc.b "BLACKHAND - ",34,"THAT'S THE SIGN OF SCEPTRE. GOOD WORK, "
                dc.b "YOU'RE INSIDE.",34,0
l_enter3:       dc.b "SARGE - ",34,"WHILE YOU'RE THERE, YOU MIGHT COME ACROSS "
                dc.b "BIOTECHNICAL MODIFICATIONS - UPGRADES - THAT IMPROVE YOUR "
                dc.b "STRENGTH AND VITALITY. IF YOU DON'T HAVE A PROBLEM WITH "
                dc.b "MESSING WITH YOUR BODY, USE THE HEALING MACHINES TO "
                dc.b "INSTALL THEM. ALSO LOOK FOR ARMOR UPGRADES.",34,0

;-------------------------------------------------------------------------------
; $0306 - SCEPTRE train platform
;-------------------------------------------------------------------------------

trainplatform:  lda #$12                        ;Disable the other trigger
                jsr setlvlobjstat
                lda #$13
                jsr setlvlobjstat
                setbit PLOT_SCEPTRE_ENTERED
                givepoints 10000
                getbit PLOT_BLACKHAND_DEAD
                jumptrue tp_abort
                jsr transmission
                plrsay l_platform1
                saytrans l_platform2
tp_abort:       stop

l_platform1:    dc.b 34, "THIS LOOKS LIKE A TRAIN PLATFORM.",34,0
l_platform2:    dc.b "BLACKHAND - ",34,"IF OUR RESEARCH IS CORRECT, THE TUNNELS CONNECT TO OTHER SCEPTRE FACILITIES. IT'S THE SO-CALLED OUTER SPHERE.",34,0



;-------------------------------------------------------------------------------
; $0308 - SCEPTRE codes
;-------------------------------------------------------------------------------

sceptrecodes:   getbit PLOT_BLACKHAND_DEAD      ;Check for bizarre possibility
                jumptrue sc_abort               ;as result of hacking..

                lda aonum                       ;Let this trigger happen
                jsr deactivateobject            ;again, and again..
                lda #PLOT_SCEPTRE_CODE1_FOUND
                sta temp1
                lda #ITEM_CODESHEET1
                sta temp2
sc_checkloop:   lda temp1                       ;Check all 4 codes
                jsr checkplotbit
                bne sc_skipcheck
                lda temp2
                jsr finditem
                bcc sc_skipcheck
                lda temp1
                jsr setplotbit
                jmp sc_newcode
sc_skipcheck:   inc temp1
                inc temp2
                lda temp1
                cmp #PLOT_SCEPTRE_CODE1_FOUND+4
                bcc sc_checkloop
sc_abort:       stop

sc_newcode:     inc codesfound
                lda codesfound
                choice 1,sc_msg1
                choice 2,sc_msg2
                choice 3,sc_msg3
                choice 4,sc_msg4                
                stop

sc_msg1:        givepoints 10000
                jsr transmission
                plrsay l_scmsg10
                saytrans l_scmsg11
                stop
sc_msg2:        givepoints 10000
                jsr transmission
                plrsay l_scmsg20
                saytrans l_scmsg21
                stop
sc_msg3:        givepoints 10000
                jsr transmission
                plrsay l_scmsg30
                saytrans l_scmsg31
                stop
sc_msg4:        givepoints 10000
                jsr transmission
                plrsay l_scmsg40
                saytrans l_scmsg41
sc_settrigger:  settrigger ACT_BLACKHAND,SCRIPT_CODESFOUND,T_NEAR|T_CONV
                lda #$03
                sta temp7
                ldx #$26
                ldy #$0f
                lda #ACT_BLACKHAND
                jsr transportactor
                ldx #$28
                ldy #$0f
                lda #ACT_SARGE
                jsr transportactor
                ldy #LA_FINE            ;Sitting
                lda #M_SIT*16+$03
                sta (lvlactptrlo),y
                stop

l_scmsg10:      dc.b 34,"I'VE FOUND SOME KIND OF A CODE.",34,0
l_scmsg11:      dc.b "BLACKHAND - ",34,"INTERESTING. I DON'T HAVE AN IDEA OF IT SO FAR, BUT KEEP LOOKING.",34,0

l_scmsg20:      dc.b 34,"ANOTHER CODE. SIMILAR TO THE FIRST.",34,0
l_scmsg21:      dc.b "BLACKHAND - ",34,"HMM.. LIKE PIECES IN A PUZZLE, PERHAPS.",34,0

l_scmsg30:      dc.b 34,"YET ANOTHER. STILL GOT ONE FACILITY TO CHECK.",34,0
l_scmsg31:      dc.b "BLACKHAND - ",34,"GOOD WORK. IT MUST BE SOMETHING SIGNIFICANT.",34,0

l_scmsg40:      dc.b 34,"NOW I HAVE CODES FROM ALL FOUR FACILITIES.",34,0
l_scmsg41:      dc.b "BLACKHAND - ",34,"EXCELLENT. AND I HAVE AN IDEA. PLEASE COME BACK TO HQ.",34,0

;-------------------------------------------------------------------------------
; $0309 - Technician boss
;-------------------------------------------------------------------------------

techboss:       choice T_APPEAR,techboss_appear
                choice T_REMOVE,techboss_remove
techboss_takedown:
                lda actt,x
                jsr removeactortrigger
                lda #ITEM_BASEMENT_ACCESS
                sta actwpn,x                    ;Drop keycard
techboss_remove:jmp playzonetune

techboss_appear:lda tunenum
                and #$fc
                ora #$03
                jmp playgametune

;-------------------------------------------------------------------------------
; $030a - Agency decoding scene shortcut
;-------------------------------------------------------------------------------

setdecodetrigger:
                jsr sc_settrigger
                jmp centerplayer

;-------------------------------------------------------------------------------
; $030b - IAC at the hangar facility
;-------------------------------------------------------------------------------

iac:            choice T_APPEAR,iac_appear
                choice T_REMOVE,iac_remove
iac_takedown:   lda actt,x
                jsr removeactortrigger
iac_remove:     lda #$0c        ;Green glow disappears
                sta zonebg3+1
                jmp playzonetune

iac_appear:     lda #$05        ;Green glow
                sta zonebg3+1
                lda tunenum
                and #$fc
                ora #$03
                jmp playgametune

                endscript

